#!/usr/bin/perl -w

use strict;

my $datadir='/var/lib/arpwatch';
my $mysqlfile='/tmp/mysqldata';
my $arg = shift;
my $verbose = 0;
$verbose = 1 if $arg && $arg eq '-d';

opendir (DIR, $datadir) or die "Unable to opendir $datadir: $!\n";
my @files;
foreach (readdir (DIR)) {
	# We want all arp.dat.* files, but not their backups
	next unless /^arp.dat/;
	next if /\-$/;
	print "$_ to be read\n" if $verbose;
	push (@files, $_);
}
chdir ($datadir);
open (MYSQL, ">$mysqlfile") or die "Unable to write $mysqlfile: $!\n";

foreach my $file (@files) {
	my ($vlan) = $file =~ /arp\.dat\.(.*)/;
	# correct for eth0
	$vlan = 'vlan8' if $vlan eq 'eth0';
	print "file $file; vlan $vlan\n" if $verbose;
	die "No vlan found in $file\n" unless $vlan;
	open (DATA, $file) or die "Unable to read $file: $!\n";
	while(<DATA>) {
		my ($mac, $ip, $time, $name) = split /\s/;
		$mac = mac2string ($mac);
		my $date = getdate ($time);
		my $subnet = getsubnet ($ip);
		print "$mac, $ip, $time, $name, $date, $subnet,$vlan\n" 
			if $verbose;
		print MYSQL "$mac,$ip,$time,$name,$date,$subnet,$vlan\n";
	}
	close (DATA);
}
close (MYSQL);

my $create = 'create table if not exists data ('.
'mac char(17), ip char (15), time int (11), name varchar(30),'.
'date char(20), subnet int (3), vlan char(10))';

my $cmd = "/usr/bin/mysql -s -e '$create' arpwatch";
print $cmd, "\n" if $verbose;
system ($cmd);

$cmd = "/usr/bin/mysql -s -e 'delete from data; load data infile \"$mysqlfile\" into table data fields terminated by \",\"' arpwatch";
print $cmd, "\n" if $verbose;
system ($cmd);

unlink ($mysqlfile);

exit 1;

#
# subroutines
#

# convert all octets in a mac address into two-char representations
sub mac2string {
	my ($mac) = @_;
	my $nmac = '';
	foreach my $oct (split /:/, $mac) {
		$oct = '0'.$oct if hex($oct) < 16;
		$nmac .= "$oct:";
	}
	chop $nmac;
	return $nmac;
}

# convert time() format to human readable
sub getdate {
	my ($time) = @_;
	my ($sec, $min, $hour, $mday, $mon, $year, $wday, @rest) =
		gmtime($time);
		$year += 1900;
	$mon++;
	$hour = '0'.$hour if $hour < 10;
	$min = '0'.$min if $min < 10;
	$mon = '0'.$mon if $mon < 10;
	$mday = '0'.$mday if $mday < 10;
	return ("$year-$mon-$mday-$hour:${min}GMT");
}

# Calculate which subnet based on IP
sub getsubnet {
	my ($ip) = @_;
	my ($a,$b,$c,$d) = split (/\./, $ip);
	return 0 unless (($a == 129) && ($b == 169));
	return 4*(int $c/4);
}
