#!/usr/bin/perl -w

use strict;
use DBD::mysql;

my $debug = 0;
my $arg = shift;
$debug = 1 if ($arg && $arg eq '-d');

# The first part of this script outputs mac addresses which have multiple IPs
# in the relevant subnets.
# The second have culls cnames on these subnets from /export/netinfo/hosts

my $hosts = '/export/netinfo/hosts';
my $dsn = 'DBI:mysql:database=arpwatch';
my $time = time ();
my $date = time2date($time);
my $cutofftime = $time - 86400*7;	# ignore entries more than a week old
my $localmac = '00:30:48:42:c5:14';	# we'll ignore the local mac, as it
					# appears on all subnets
my @subnets = qw /8 12 16/;		# subnets to match in MAC data
my @prefixes = qw /8 9 10 11 12 13 14 15 16 17 18 19/; # IP prefixes to match
my $outfile = '/usr/local/www/htdocs/servicemachines.html';
unless ($debug) {
	open (OUT, ">$outfile") or die "Unable to write $outfile: $!\n";
	select (OUT);
}

my $dbh = DBI->connect($dsn);
my $sth = $dbh->prepare('SELECT ip,mac,name FROM data WHERE subnet=? AND TIME>='.$cutofftime);

print "<pre>\n";
print "<h4>Generated $date</h4>\n";
foreach my $subnet (@subnets) {
	my %RApairs;	# array of pairs of ip,name indexed by mac
	$sth->execute ($subnet) 
		or return ["Error in select: ", $sth->errstr];
	while (my ($ip, $mac, $name) = $sth->fetchrow_array()) {
#		print "$ip $mac $name\n";
		push (@{$RApairs{$mac}}, "$ip\t$name");
	}
	foreach my $mac (keys %RApairs) {
		next if $mac eq $localmac;
		next unless scalar @{$RApairs{$mac}} > 1;
		foreach my $row (@{$RApairs{$mac}}) {
			print "$mac\t$row\n";
		}
		print "\n";
	}
}

open (HOSTS, $hosts) or die "Unable to open $hosts: $!\n";
while (<HOSTS>) {
	next unless /^129\.169\.(\d+)(\.\d+)\s+(\S+\s+\S+.*)$/;
	my ($prefix, $postfix, $rest) = ($1, $2, $3);
	next unless scalar grep { /^$prefix$/ } @prefixes;
	print "129.169.$prefix$postfix\t";
	my (@names) = split (/ /, $rest);
	while (scalar @names > 4) {
		for (my $i=0; $i<4; $i++) {
			print shift (@names), " ";
		}
		print "\n\t\t";
	}
	print join (' ', @names), "\n";
}
print "</pre>\n";

exit 0;

#
# subroutines
#

sub time2date {
	my ($time) = @_;
	my ($sec,$min,$hour,$mday,$mon,$year,@rest) = gmtime($time);
	$year +=1900;
	$mon++;
	$mon = '0.'.$mon if $mon < 10;
	$mday = '0'.$mday if $mday < 10;
	return "$year-$mon-$mday";
}
